/*
   Bug fixed MENU_REXT demo

   We need a function to USE_THG because MENU_REXT requires a parameter in a 
   register; c68 does not offer this facility (yet).
        (*func)(__A1 parameter) 

   You could link without -lhotthg; but only if you like system
   crashes (the v4.14 c68 routines have some bugs), later versions
   are probably fixed.

   It is also necessary to correct <things.h> THING_HEADER_EXT definition
   (remove extra thh_next, thh_exld to give:

    typedef struct
    {
        THING_HEADER_LIST thh;
        long thh_pdef;
        long thh_pdes;
        char thh_code[1];
    } THING_HEADER_EXT;
  
    Likewise, later versions are probably fixed.

*/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qdos.h>
#include <qptr.h>
#include <things.h>
#include <unistd.h>
#include "qspv-proto.h"
 
typedef struct
{
    short attr;
    short info;
    void *ptr;
} MENUP;

typedef struct
{
    MENUP   mennm;
    MENUP   defnm;
    MENUP   dirnm;
    MENUP   extnm;
    long    size;
    long    xpos;
    long    ypos;
    long    manic;
    long    filec;
    MENUP   fname;
} MENUFS;

#define THP_UPD     0xe000
#define THP_CALL    0xc000
#define THP_RET     0xa000
#define THP_PTR     0x8000
#define THP_OPT     0x1000

#define THP_CHAR    0x0004
#define THP_UBYT    0x0008
#define THP_SBYT    0x000a
#define THP_UWRD    0x0010
#define THP_SWRD    0x0012
#define THP_ULNG    0x0020
#define THP_SLNG    0x0022
#define THP_FP8     0x0042
#define THP_STR     0x0100
#define THP_SSTR    0x0200

#define MOVEIN(s, t) s.len = strlen(t); strcpy((char *)s.chrs, t)
#define PACK(a,b,c,d) (d + (c << 8) + (b << 16) + (a << 24))

/*
 * This is our USE_THG routine
 */
 
static const short
      use_thg [] = {0x206F, 0x0004,     /*      move.l  4(sp),a0 */
                    0x226F, 0x0008,     /*      move.l  8(sp),a1 */
                    0x4E90,             /*      jsr     (a0)     */
                    0x4E75};            /*      rts              */

static void restore(char *buf, struct QD_text *qdt)
{
    strncpy(buf, (char*)qdt->chrs, qdt->len);
    *(buf + qdt->len) = '\0';
}

char * mygetcwd(char * nambuf, int len)
{
    extern char *_sys_var;
    char *s = NULL;
    struct QLSTR **q;
    
    if(q = (struct QLSTR **q)(_sys_var + 0xAC))
    {
        
        q++;
        if(len > (*q)->qs_strlen) 
        {
            if(nambuf == NULL)
            {
                s = malloc(len);
            }
            else
            {
                s = nambuf;
            }
            strncpy(s, (*q)->qs_str, len);
            *(s+(*q)->qs_strlen) = 0;
        }
    }
    return s;
}

int GetMenuFile (char *prompt, char *dirs, char *buf)
{
    int i;
    long vers;

    /*
     * We must ensure this is on an even boundary
     */
     
    static const long fsel = PACK('F','S','E','L');
    void *ps = NULL;
    char *p;
    THING_HEADER_EXT *pThing;
    
    static QD_TEXT(39) menustr;
    static QD_TEXT(39) basestr;
    static QD_TEXT(39) direstr;
    static QD_TEXT(39) rsltstr;
    static QD_TEXT(39) extstr;
    
    static MENUFS s =
        {
           {THP_CALL|THP_OPT|THP_STR, 0, &menustr},
           {THP_CALL|THP_OPT|THP_STR, 0, &basestr},
           {THP_CALL|THP_OPT|THP_STR|THP_UPD, 0, &direstr},
           {THP_CALL|THP_OPT|THP_STR, 0, &extstr},
           0,
           0,
           0,
           -1,
           -1,
           {THP_RET|THP_STR, 0, &rsltstr}
        };

    MOVEIN (menustr, prompt);
    MOVEIN (basestr, buf);  

    if(p = strrchr(dirs, '_'))
    {
        char *q;
        char c;

	if(p == dirs)
	{
	    char def[40];
	    mygetcwd(def, 39);
	    MOVEIN (direstr, def);
	}
	else
	{
	    if(p[-1] != '_')
	    {
		p++;
	    }
	    c = *p;
	    *p = '\0';
	    MOVEIN (direstr, dirs);
	    *p = c;
	}
        MOVEIN (extstr, p);
    }        

    rsltstr.len = 0;
    
    i = sms_uthg("Menus", -1, -1, (long *) &fsel, NULL, &vers, &ps);
    if(i > 0)
    {
        int (*func)(void *, MENUFS *) = (int(*)())use_thg;
        pThing = (THING_HEADER_EXT *) i;
/*
 * c68 doesn't allow   ((int(*)()) use_thg) (pThing->thh_code, &s);
 * where use_thg isn't really a function.
 */
        i = (func) (pThing->thh_code, &s);
        if(i >= 0)
        {
            restore(buf,  (struct QD_text *) &rsltstr);
            restore(dirs, (struct QD_text *) &direstr);
        }
    } 
    if(i < 0)
    {
	QErr ("Menu_rext", i);
    }
    return i;
}

char *stpcpy(char *d, const char *s)
{
    while (*d++ = *s++);
    return(d-1);
}
